'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

(function () {
  var Event,
      Exp,
      Tree,
      hasProp = {}.hasOwnProperty;

  Exp = require('../data-bind/exp');

  Event = require('../dom/core/event');

  module.exports = Tree = function () {
    var classRegExp, directiveRegExp, eventRegExp, linkRegExp, propRegExp;

    var Tree = function () {
      function Tree(template1, Component) {
        _classCallCheck(this, Tree);

        var hasTerminal;
        this.template = template1;
        this.Component = Component;
        if (this.template.nodeType === 'text') {
          this.isTextNode = true;
          this.hasTextExp = Exp.test(this.template.value);
          return;
        }
        this.classes = {};
        this.events = {};
        this.props = {};
        this.links = [];
        this.directives = [];
        this.childTrees = [];
        this.expAttributes = {};
        hasTerminal = this.compileDirectives();
        if (hasTerminal) {
          return;
        }
        this.SubComponent = this.Component.getSubComponent(this.template);
        this.compileAttributes();
        this.compileChildTrees();
        return;
      }

      _createClass(Tree, [{
        key: 'create',
        value: function create(template) {
          return new Tree(template, this.Component);
        }
      }, {
        key: 'replace',
        value: function replace(node) {
          this.template.replace(node);
          this.template = node;
        }
      }, {
        key: 'compileDirectives',
        value: function compileDirectives() {
          var _this = this;

          var directives, hasTerminal;
          if (this.template.nodeType !== 'element') {
            return;
          }
          directives = this.getDirectives();
          directives = this.sortDirectives(directives);
          directives = this.filterDirectives(directives);
          hasTerminal = this.directivesHasTerminal(directives);
          this.directives = directives.map(function (directive) {
            var returns;
            returns = typeof directive.compile === "function" ? directive.compile(_this.template, _this) : void 0;
            return function (node, component, scope) {
              if (returns == null) {
                returns = [];
              }
              return new (Function.prototype.bind.apply(directive, [null].concat([node, component, scope], _toConsumableArray(returns))))();
            };
          });
          return hasTerminal;
        }
      }, {
        key: 'getDirectives',
        value: function getDirectives() {
          var _this2 = this;

          var directives;
          directives = [];
          this.template.eachAttrs(function (name, value) {
            var Directive, directive;
            if (directiveRegExp.test(name)) {
              directive = name.match(directiveRegExp)[1];
              Directive = _this2.Component.getDirective(directive);
              if (Directive) {
                return directives.push(Directive);
              }
            }
          });
          return directives;
        }
      }, {
        key: 'sortDirectives',
        value: function sortDirectives(directives) {
          directives.sort(function (a, b) {
            return a.priority < b.priority;
          });
          return directives;
        }
      }, {
        key: 'filterDirectives',
        value: function filterDirectives(directives) {
          var directive, filtered, i, len;
          filtered = [];
          for (i = 0, len = directives.length; i < len; i++) {
            directive = directives[i];
            filtered.push(directive);
            if (directive.terminal) {
              break;
            }
          }
          return filtered;
        }
      }, {
        key: 'directivesHasTerminal',
        value: function directivesHasTerminal(directives) {
          return directives.some(function (directive) {
            return directive.terminal;
          });
        }
      }, {
        key: 'compileAttributes',
        value: function compileAttributes() {
          var _this3 = this;

          if (this.template.nodeType !== 'element') {
            return;
          }
          this.template.eachAttrs(function (name, value) {
            var className, event, link, matches, prop;
            if (eventRegExp.test(name)) {
              matches = name.match(eventRegExp);
              event = matches[3];
              return _this3.events[event] = {
                own: !!matches[1],
                prevent: !!matches[2],
                exp: value
              };
            } else if (classRegExp.test(name)) {
              className = name.match(classRegExp)[1];
              return _this3.classes[className] = value;
            } else if (propRegExp.test(name)) {
              prop = name.match(propRegExp)[1];
              return _this3.props[prop] = value;
            } else if (linkRegExp.test(name)) {
              link = name.match(linkRegExp)[1];
              return _this3.links.push(link);
            } else if (Exp.test(value)) {
              return _this3.expAttributes[name] = value;
            }
          });
        }
      }, {
        key: 'compileChildTrees',
        value: function compileChildTrees() {
          var child, i, len, ref;
          ref = this.template.children;
          for (i = 0, len = ref.length; i < len; i++) {
            child = ref[i];
            this.childTrees.push(new Tree(child, this.Component));
          }
        }
      }, {
        key: 'init',
        value: function init(node, component) {
          var _this4 = this;

          var scope = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

          var subComponent;
          if (this.isTextNode) {
            if (this.hasTextExp) {
              this.initTextExp(node, component, scope);
            }
            return component;
          }
          if (this.SubComponent) {
            subComponent = this.SubComponent.init(node, component.app, this, component, scope, function (instance) {
              _this4.initLinks(instance, scope);
              return _this4.initProps(instance, component, scope);
            });
          } else {
            this.initLinks(node, scope);
            this.initProps(node, component, scope);
          }
          this.initClasses(node, component, scope);
          this.initExpAttributes(node, component, scope);
          this.initEvents(node, component, scope, subComponent);
          this.initChildren(node, component, scope);
          this.initDirectives(node, component, scope);
          return component;
        }
      }, {
        key: 'initChildren',
        value: function initChildren(node, component, scope) {
          var child, childNode, childNodes, i, index, len, ref;
          childNodes = node.children.slice();
          ref = this.childTrees;
          for (index = i = 0, len = ref.length; i < len; index = ++i) {
            child = ref[index];
            childNode = childNodes[index];
            child.init(childNode, component, scope);
          }
        }
      }, {
        key: 'initDirectives',
        value: function initDirectives(node, component, scope) {
          var directive, fn, i, len, ref;
          ref = this.directives;
          fn = function fn(directive) {
            var dir;
            dir = directive(node, component, scope);
            return node.on('destroy', function () {
              return typeof dir.destructor === "function" ? dir.destructor() : void 0;
            });
          };
          for (i = 0, len = ref.length; i < len; i++) {
            directive = ref[i];
            fn(directive);
          }
        }
      }, {
        key: 'initLinks',
        value: function initLinks(target, scope) {
          var i, len, link, ref;
          ref = this.links;
          for (i = 0, len = ref.length; i < len; i++) {
            link = ref[i];
            scope[link] = target;
          }
        }
      }, {
        key: 'initEvents',
        value: function initEvents(node, component, scope, subComponent) {
          var eventData, eventName, fn, ref;
          ref = this.events;
          fn = function fn(eventData) {
            return node.on(eventName, function (event) {
              if (event instanceof Event) {
                if (eventData.prevent) {
                  event.prevent();
                  event.stop();
                }
                if (eventData.own && !event.own) {
                  return;
                }
              }
              if (eventData.exp) {
                scope['$event'] = event;
                scope['this'] = subComponent || node;
                ui.eval(component, eventData.exp, scope, false);
                delete scope['$event'];
                delete scope['this'];
              }
            });
          };
          for (eventName in ref) {
            if (!hasProp.call(ref, eventName)) continue;
            eventData = ref[eventName];
            fn(eventData);
          }
        }
      }, {
        key: 'initClasses',
        value: function initClasses(node, component, scope) {
          var className, exp, fn, ref;
          ref = this.classes;
          fn = function fn(className, exp) {
            var bind;
            bind = node.bindClass(className, exp, component, scope);
            return node.on('destroy', function () {
              return bind.destroy();
            });
          };
          for (className in ref) {
            if (!hasProp.call(ref, className)) continue;
            exp = ref[className];
            fn(className, exp);
          }
        }
      }, {
        key: 'initProps',
        value: function initProps(target, component, scope) {
          var exp, fn, prop, ref;
          ref = this.props;
          fn = function fn(prop, exp) {
            var bind;
            bind = ui.bind(target, prop, component, exp, scope);
            return target.on('destroy', function () {
              return bind.destroy();
            });
          };
          for (prop in ref) {
            if (!hasProp.call(ref, prop)) continue;
            exp = ref[prop];
            fn(prop, exp);
          }
        }
      }, {
        key: 'initTextExp',
        value: function initTextExp(node, component, scope) {
          var exp, watcher;
          exp = node.value;
          watcher = ui.watch(component, exp, function (value) {
            return node.value = value;
          }, scope);
          node.on('destroy', function () {
            return watcher.destroy();
          });
        }
      }, {
        key: 'initExpAttributes',
        value: function initExpAttributes(node, component, scope) {
          var exp, fn, name, ref;
          ref = this.expAttributes;
          fn = function fn(name) {
            var watcher;
            watcher = ui.watch(component, exp, function (value) {
              return node.attr(name, value);
            }, scope);
            return node.on('destroy', function () {
              return watcher.destroy();
            });
          };
          for (name in ref) {
            if (!hasProp.call(ref, name)) continue;
            exp = ref[name];
            fn(name);
          }
        }
      }]);

      return Tree;
    }();

    ;

    eventRegExp = /^\((\()?(!?)(.+?)\)?\)$/;

    directiveRegExp = /^\*(\S+)$/;

    propRegExp = /^\[(\S+)]$/;

    linkRegExp = /^#(\S+)$/;

    classRegExp = /^\.(\S+)$/;

    return Tree;
  }();
}).call(undefined);