'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

(function () {
  var CallPropObserver, Exp, ExpPropObserver, PathObserver, PropObserver, equals;

  PropObserver = require('./prop-observer');

  Exp = require('./exp');

  equals = function equals(a, b) {
    if (a !== a && b !== b) {
      return true;
    }
    return a === b;
  };

  module.exports = PathObserver = function () {
    function PathObserver(context1, path, handler1) {
      var _this = this;

      var rootContext1 = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : this.context;

      _classCallCheck(this, PathObserver);

      this.context = context1;
      this.handler = handler1;
      this.rootContext = rootContext1;
      if (this.context !== Object(this.context)) {
        return;
      }
      this.destroyed = false;
      this.path = path.slice();
      this.accessor = this.path.shift();
      this.accessorObserver = null;
      this.subPathObserver = null;
      this.value = void 0;
      if (!this.path.length) {
        this.observeAccessor(this.handler);
      } else {
        this.initSubPathObserver();
        this.observeAccessor(function (newSubContext) {
          return _this.updateSubPathObserver(newSubContext);
        });
      }
      return;
    }

    _createClass(PathObserver, [{
      key: 'observeAccessor',
      value: function observeAccessor(handler) {
        this.accessorObserver = function () {
          switch (this.accessor.type) {
            case 'prop':
              return this.createPropObserver(handler);
            case 'exp':
              return this.createExpPropObserver(handler);
            case 'call':
              return this.createCallPropObserver(handler);
          }
        }.call(this);
      }
    }, {
      key: 'createPropObserver',
      value: function createPropObserver(handler) {
        var name;
        name = this.accessor.name.toString();
        return new PropObserver(this.context, name, handler);
      }
    }, {
      key: 'createExpPropObserver',
      value: function createExpPropObserver(handler) {
        return new ExpPropObserver(this.rootContext, this.context, this.accessor, handler);
      }
    }, {
      key: 'createCallPropObserver',
      value: function createCallPropObserver(handler) {
        return new CallPropObserver(this.rootContext, this.context, this.accessor, handler);
      }
    }, {
      key: 'initSubPathObserver',
      value: function initSubPathObserver() {
        var subContext;
        subContext = this.getByAccessor(this.context, this.accessor);
        this.subPathObserver = new PathObserver(subContext, this.path, this.handler, this.rootContext);
        this.value = this.getByPath(subContext, this.path);
      }
    }, {
      key: 'updateSubPathObserver',
      value: function updateSubPathObserver(newSubContext) {
        var oldValue;
        this.subPathObserver.destroy();
        this.subPathObserver = new PathObserver(newSubContext, this.path, this.handler, this.rootContext);
        oldValue = this.value;
        this.value = this.getByPath(newSubContext, this.path);
        if (!equals(this.value, oldValue)) {
          this.handler(this.value);
        }
      }
    }, {
      key: 'destroy',
      value: function destroy() {
        var ref, ref1;
        if (this.destroyed) {
          return;
        }
        if ((ref = this.subPathObserver) != null) {
          ref.destroy();
        }
        if ((ref1 = this.accessorObserver) != null) {
          ref1.destroy();
        }
        this.destroyed = true;
      }
    }, {
      key: 'getByAccessor',
      value: function getByAccessor(context, accessor) {
        switch (accessor.type) {
          case 'prop':
            return context[accessor.name];
          case 'exp':
            return ExpPropObserver.getValue(this.rootContext, context, accessor);
          case 'call':
            return CallPropObserver.getValue(this.rootContext, context, accessor);
        }
      }
    }, {
      key: 'getByPath',
      value: function getByPath(context, path) {
        var accessor, i, len;
        for (i = 0, len = path.length; i < len; i++) {
          accessor = path[i];
          if (context == null) {
            return void 0;
          }
          context = this.getByAccessor(context, accessor);
        }
        return context;
      }
    }]);

    return PathObserver;
  }();

  ExpPropObserver = function () {
    _createClass(ExpPropObserver, null, [{
      key: 'getValue',
      value: function getValue(rootContext, context, accessor) {
        var exp, key;
        exp = new Exp(accessor.exp.code);
        key = exp(rootContext);
        return context[key];
      }
    }]);

    function ExpPropObserver(rootContext, context, accessor, handler) {
      var _this2 = this;

      _classCallCheck(this, ExpPropObserver);

      var ExpObserver, exp;
      ExpObserver = require('./exp-observer');
      this.destroyed = false;
      this.oldValue = void 0;
      this.propObserver = null;
      exp = accessor.exp.code;
      this.expObserver = new ExpObserver(rootContext, exp, function (expValue) {
        var ref;
        if ((ref = _this2.propObserver) != null) {
          ref.destroy();
        }
        return _this2.propObserver = new PropObserver(context, expValue, function (value) {
          if (!equals(value, _this2.oldValue)) {
            _this2.oldValue = value;
            return handler(value);
          }
        }, true);
      }, null, true);
      return;
    }

    _createClass(ExpPropObserver, [{
      key: 'destroy',
      value: function destroy() {
        var ref;
        if (this.destroyed) {
          return;
        }
        if ((ref = this.propObserver) != null) {
          ref.destroy();
        }
        this.expObserver.destroy();
        this.destroyed = true;
      }
    }]);

    return ExpPropObserver;
  }();

  CallPropObserver = function () {
    _createClass(CallPropObserver, null, [{
      key: 'getValue',
      value: function getValue(rootContext, context, accessor) {
        var args, argsExp, name;
        name = accessor.name.toString();
        argsExp = new Exp('[' + accessor.exp.code + ']');
        args = argsExp(rootContext);
        return context[name].apply(context, args);
      }
    }]);

    function CallPropObserver(rootContext1, context1, accessor1, handler1) {
      _classCallCheck(this, CallPropObserver);

      var ExpObserver, argsExp, name;
      this.change = this.change.bind(this);
      this.rootContext = rootContext1;
      this.context = context1;
      this.accessor = accessor1;
      this.handler = handler1;
      ExpObserver = require('./exp-observer');
      this.destroyed = false;
      this.oldValue = void 0;
      name = this.accessor.name.toString();
      this.funcObserver = new PropObserver(this.context, name, this.change);
      argsExp = '[' + this.accessor.exp.code + ']';
      this.argsObserver = new ExpObserver(this.rootContext, argsExp, this.change);
      return;
    }

    _createClass(CallPropObserver, [{
      key: 'change',
      value: function change() {
        var value;
        value = CallPropObserver.getValue(this.rootContext, this.context, this.accessor);
        if (!equals(value, this.oldValue)) {
          this.oldValue = value;
          this.handler(value);
        }
      }
    }, {
      key: 'destroy',
      value: function destroy() {
        if (this.destroyed) {
          return;
        }
        this.funcObserver.destroy();
        this.argsObserver.destroy();
        this.destroyed = true;
      }
    }]);

    return CallPropObserver;
  }();
}).call(undefined);