'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

(function () {
  var Element,
      ExpObserver,
      Node,
      Selector,
      ShadowRoot,
      Style,
      indexOf = [].indexOf,
      hasProp = {}.hasOwnProperty;

  Node = require('./node');

  Selector = require('../core/selector');

  ExpObserver = require('../../data-bind/exp-observer');

  Style = require('../core/style');

  module.exports = Element = function () {
    var Element = function (_Node) {
      _inherits(Element, _Node);

      function Element(tag, attrs) {
        _classCallCheck(this, Element);

        var _this = _possibleConstructorReturn(this, (Element.__proto__ || Object.getPrototypeOf(Element)).call(this));

        _this.tag = tag.toLowerCase();
        _this.nodeType = 'element';
        _this.style = new Style(_this);
        _this.attrs = {};
        _this.attrNames = [];
        _this.classes = [];
        _this.shadowRoot = null;
        if (attrs) {
          _this.attr(attrs);
        }
        return _possibleConstructorReturn(_this);
      }

      _createClass(Element, [{
        key: 'calcStyle',
        value: function calcStyle() {
          if (!this.realNode) {
            return null;
          }
          return getComputedStyle(this.realNode);
        }
      }, {
        key: 'toEm',
        value: function toEm(px) {
          var fontSize, style;
          style = this.calcStyle();
          if (!style) {
            return 0;
          }
          fontSize = parseFloat(style.fontSize);
          return px / fontSize;
        }
      }, {
        key: 'toPx',
        value: function toPx(em) {
          var fontSize, style;
          style = this.calcStyle();
          if (!style) {
            return 0;
          }
          fontSize = parseFloat(style.fontSize);
          return em * fontSize;
        }
      }, {
        key: 'renderStyle',
        value: function renderStyle() {
          var style;
          if (!this.realNode) {
            return null;
          }
          style = this.style.toString();
          this.realNode.setAttribute('style', style);
          this.realNode.offsetHeight;
        }
      }, {
        key: 'own',
        value: function own(eventName, handler) {
          this.on(eventName, function (event) {
            if (event.own) {
              handler(event);
            }
          });
        }
      }, {
        key: 'in',
        value: function _in(element) {
          var context;
          context = this;
          while (context) {
            if (context.parent === element) {
              return true;
            }
            context = context.parent;
          }
          return false;
        }
      }, {
        key: 'width',
        value: function width() {
          var toEm = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

          var width;
          if (!this.realNode) {
            return 0;
          }
          width = this.realNode.offsetWidth;
          if (toEm) {
            width = this.toEm(width);
          }
          return width;
        }
      }, {
        key: 'height',
        value: function height() {
          var toEm = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

          var height;
          if (!this.realNode) {
            return 0;
          }
          height = this.realNode.offsetHeight;
          if (toEm) {
            height = this.toEm(height);
          }
          return height;
        }
      }, {
        key: 'rect',
        value: function rect() {
          var rect;
          if (!this.realNode) {
            return {
              width: 0,
              height: 0,
              left: 0,
              right: 0,
              top: 0,
              bottom: 0
            };
          }
          rect = this.realNode.getBoundingClientRect();
          return {
            width: rect.width,
            height: rect.height,
            left: rect.left,
            right: rect.right,
            top: rect.top,
            bottom: rect.bottom
          };
        }
      }, {
        key: 'bindClass',
        value: function bindClass(className, exp, context, scope) {
          var _this2 = this;

          return new ExpObserver(context, exp, function (value) {
            if (value) {
              return _this2.addClass(className);
            } else {
              return _this2.removeClass(className);
            }
          }, scope, true);
        }
      }, {
        key: 'addClass',
        value: function addClass(className) {
          if (indexOf.call(this.classes, className) < 0) {
            this.classes.push(className);
            this.attr('class', this.classes.join(' '));
          }
        }
      }, {
        key: 'removeClass',
        value: function removeClass(className) {
          var index;
          index = this.classes.indexOf(className);
          if (index !== -1) {
            this.classes.splice(index, 1);
            if (this.classes.length) {
              this.attr('class', this.classes.join(' '));
            } else {
              this.removeAttr('class');
            }
          }
        }
      }, {
        key: 'hasClass',
        value: function hasClass(className) {
          return this.classes.indexOf(className) !== -1;
        }
      }, {
        key: 'toggleClass',
        value: function toggleClass(className) {
          if (this.hasClass(className)) {
            this.removeClass(className);
          } else {
            this.addClass(className);
          }
        }
      }, {
        key: 'css',
        value: function css(rules) {
          var prop, value;
          if (arguments.length === 2) {
            this.style[arguments[0]] = arguments[1];
            return;
          }
          for (prop in rules) {
            if (!hasProp.call(rules, prop)) continue;
            value = rules[prop];
            this.style[prop] = value;
          }
        }
      }, {
        key: 'renderCss',
        value: function renderCss() {
          this.css.apply(this, arguments);
          this.renderStyle();
        }
      }, {
        key: 'focus',
        value: function focus() {
          var ref;
          if ((ref = this.realNode) != null) {
            ref.focus();
          }
        }
      }, {
        key: 'attr',
        value: function attr(name, value) {
          var index, key;
          if (arguments.length === 1) {
            if ((typeof name === 'undefined' ? 'undefined' : _typeof(name)) !== 'object') {
              return this.attrs[name];
            }
            for (key in name) {
              value = name[key];
              this.attr(key, value);
            }
            return;
          }
          this.attrs[name] = value;
          index = this.attrNames.indexOf(name);
          if (index === -1) {
            this.attrNames.push(name);
          }
          if (name === 'class') {
            this.classes = (value + '').split(/\s+/);
          }
          this.mutate('changeAttrs');
          return value;
        }
      }, {
        key: 'hasAttr',
        value: function hasAttr(name) {
          return this.attrs.hasOwnProperty(name);
        }
      }, {
        key: 'removeAttr',
        value: function removeAttr(name) {
          var index;
          delete this.attrs[name];
          index = this.attrNames.indexOf(name);
          if (index !== -1) {
            this.attrNames.splice(index, 1);
          }
          if (name === 'class') {
            this.classes = [];
          }
          this.mutate('changeAttrs');
        }
      }, {
        key: 'eachAttrs',
        value: function eachAttrs(handler) {
          var attrName, i, len, ref;
          ref = this.attrNames;
          for (i = 0, len = ref.length; i < len; i++) {
            attrName = ref[i];
            handler(attrName, this.attrs[attrName]);
          }
        }
      }, {
        key: 'clone',
        value: function clone() {
          var child, element, i, len, ref;
          element = new this.constructor(this.tag, this.attrs);
          ref = this.children;
          for (i = 0, len = ref.length; i < len; i++) {
            child = ref[i];
            element.append(child.clone());
          }
          if (this.shadowRoot) {
            element.shadowRoot = this.shadowRoot.clone(element);
          }
          return element;
        }
      }, {
        key: 'html',
        value: function html(_html) {
          var Dom, child, i, len, newChildren, ref;
          if (_html instanceof Array) {
            newChildren = _html;
          } else {
            Dom = require('../index');
            newChildren = Dom.parse(_html);
          }
          this.empty();
          ref = newChildren.slice();
          for (i = 0, len = ref.length; i < len; i++) {
            child = ref[i];
            this.append(child);
          }
        }
      }, {
        key: 'createShadowRoot',
        value: function createShadowRoot(shadowId) {
          var ref;
          if ((ref = this.shadowRoot) != null) {
            ref.destroy();
          }
          this.shadowRoot = new ShadowRoot(this, shadowId);
          return this.shadowRoot;
        }
      }, {
        key: 'select',
        value: function select(selector) {
          return new Selector(selector).select(this);
        }
      }, {
        key: 'selectOne',
        value: function selectOne(selector) {
          return this.select(selector)[0];
        }
      }]);

      return Element;
    }(Node);

    ;

    Object.defineProperty(Element.prototype, 'realNode', {
      get: function get() {
        return ui.getRealNode(this);
      },
      enumerable: true
    });

    return Element;
  }();

  ShadowRoot = function (_Element) {
    _inherits(ShadowRoot, _Element);

    function ShadowRoot(host1) {
      var shadowId1 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Math.random().toString().slice(2);

      _classCallCheck(this, ShadowRoot);

      var _this3 = _possibleConstructorReturn(this, (ShadowRoot.__proto__ || Object.getPrototypeOf(ShadowRoot)).call(this, '[[shadow-root]]'));

      _this3.host = host1;
      _this3.shadowId = shadowId1;
      return _possibleConstructorReturn(_this3);
    }

    _createClass(ShadowRoot, [{
      key: 'clone',
      value: function clone(host) {
        var child, i, len, ref, shadowRoot;
        shadowRoot = new ShadowRoot(host, this.shadowId);
        ref = this.children;
        for (i = 0, len = ref.length; i < len; i++) {
          child = ref[i];
          shadowRoot.append(child.clone());
        }
        return shadowRoot;
      }
    }]);

    return ShadowRoot;
  }(Element);
}).call(undefined);