'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _arrayObserver = require('../data-bind/array-observer');

var _arrayObserver2 = _interopRequireDefault(_arrayObserver);

var _htmlRenderer = require('./html-renderer');

var _htmlRenderer2 = _interopRequireDefault(_htmlRenderer);

var _selection = require('../core/selection');

var _selection2 = _interopRequireDefault(_selection);

var _event = require('../dom/core/event');

var _event2 = _interopRequireDefault(_event);

var _frame = require('../polyfill/frame');

var _frame2 = _interopRequireDefault(_frame);

var _dom = require('../dom');

var _dom2 = _interopRequireDefault(_dom);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var DOMRenderer,
    allEventNames,
    mouseEventNames,
    touchEventNames,
    indexOf = [].indexOf;

mouseEventNames = ['mousedown', 'mousemove', 'mouseup', 'click'];

touchEventNames = ['touchstart', 'touchmove', 'touchend'];

allEventNames = [].concat(_toConsumableArray(mouseEventNames), _toConsumableArray(touchEventNames));

Object.getOwnPropertyNames(Document.prototype).forEach(function (key) {
  var eventName;
  if (key.indexOf('on') !== 0) {
    return;
  }
  eventName = key.slice(2);
  if (indexOf.call(allEventNames, eventName) >= 0) {
    return;
  }
  allEventNames.push(eventName);
});

module.exports = DOMRenderer = function () {
  function DOMRenderer(root) {
    var rootReal = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : document.body;

    _classCallCheck(this, DOMRenderer);

    this.init = this.init.bind(this);
    this.frame = this.frame.bind(this);
    this.onMutations = this.onMutations.bind(this);
    this.needsToUpdateHTML = this.needsToUpdateHTML.bind(this);
    this.updateInnerHTML = this.updateInnerHTML.bind(this);
    this.syncChildren = this.syncChildren.bind(this);
    this.renderSplice = this.renderSplice.bind(this);
    this.getChildSplices = this.getChildSplices.bind(this);
    this.getChildNodesArr = this.getChildNodesArr.bind(this);
    this.toUniqueString = this.toUniqueString.bind(this);
    this.updateAttrs = this.updateAttrs.bind(this);
    this.updateInput = this.updateInput.bind(this);
    this.initEventHandlers = this.initEventHandlers.bind(this);
    this.onDOMEvent = this.onDOMEvent.bind(this);
    this.touchToMouseEvent = this.touchToMouseEvent.bind(this);
    this.touchEndOnTheSameNode = this.touchEndOnTheSameNode.bind(this);
    this.emitVirtualEvent = this.emitVirtualEvent.bind(this);
    this.updateVirtualInput = this.updateVirtualInput.bind(this);
    this.initNode = this.initNode.bind(this);
    this.getRealNode = this.getRealNode.bind(this);
    this.getVirtualNode = this.getVirtualNode.bind(this);
    this.root = root;
    this.rootReal = rootReal;
    this.selection = new _selection2.default();
    this.observer = this.root.observe();
    this.isMobile = !!ui.isMobile();
    this.lastTouchRealNode = null;
    this.startTouchRealNode = null;
    return;
  }

  _createClass(DOMRenderer, [{
    key: 'init',
    value: function init() {
      this.rootReal.innerHTML = _htmlRenderer2.default.render(this.root);
      this.initEventHandlers();
      this.initNode(this.root);
      this.frame();
    }
  }, {
    key: 'frame',
    value: function frame() {
      ui.frame(this.frame);
      if (this.observer.hasMutations()) {
        this.selection.save();
        this.onMutations(this.observer.getMutations());
        this.selection.restore();
      }
    }
  }, {
    key: 'onMutations',
    value: function onMutations(mutations) {
      var _this = this;

      var i, len, node, ref;
      ref = this.needsToUpdateHTML(mutations);
      for (i = 0, len = ref.length; i < len; i++) {
        node = ref[i];
        this.updateInnerHTML(node);
        this.initNode(node);
      }
      mutations.forEach(function (mutation, node) {
        if (mutation.changeAttrs) {
          _this.updateAttrs(node);
        }
        if (mutation.changeInput) {
          return _this.updateInput(node);
        }
      });
    }
  }, {
    key: 'needsToUpdateHTML',
    value: function needsToUpdateHTML(mutations) {
      var nodes, unique;
      nodes = [];
      mutations.forEach(function (mutation, node) {
        if (!mutation.changeChildren && node.nodeType !== 'text') {
          return;
        }
        if (node.nodeType === 'text') {
          node = node.parent;
        }
        if (node.tag === '[[shadow-root]]') {
          node = node.host;
        }
        nodes.push(node);
      });
      nodes = nodes.filter(function (node) {
        var context;
        context = node.parent || node.host;
        while (context) {
          if (indexOf.call(nodes, context) >= 0) {
            return false;
          }
          context = context.parent || context.host;
        }
        return true;
      });
      unique = [];
      nodes.forEach(function (node) {
        if (unique.indexOf(node) === -1) {
          unique.push(node);
        }
      });
      return unique;
    }
  }, {
    key: 'updateInnerHTML',
    value: function updateInnerHTML(node) {
      var newNode, realNode;
      realNode = this.getRealNode(node);
      if (!realNode) {
        return;
      }
      newNode = document.createElement('div');
      newNode.innerHTML = _htmlRenderer2.default.renderInnerHTML(node);
      this.syncChildren(realNode, newNode);
    }
  }, {
    key: 'syncChildren',
    value: function syncChildren(oldNode, newNode) {
      var child, i, index, j, len, len1, newChildNodes, oldChildNodes, ref, splice, splices;
      oldChildNodes = [].slice.call(oldNode.childNodes);
      newChildNodes = [].slice.call(newNode.childNodes);
      splices = this.getChildSplices(oldChildNodes, newChildNodes);
      for (i = 0, len = splices.length; i < len; i++) {
        splice = splices[i];
        this.renderSplice(splice, oldNode, newChildNodes);
      }
      ref = oldNode.childNodes;
      for (index = j = 0, len1 = ref.length; j < len1; index = ++j) {
        child = ref[index];
        if (indexOf.call(newChildNodes, child) >= 0) {
          continue;
        }
        this.syncChildren(child, newChildNodes[index]);
      }
    }
  }, {
    key: 'renderSplice',
    value: function renderSplice(splice, oldNode, newChildNodes) {
      var fragment, i, index, j, oldChildNodes, ref, ref1, ref2, ref3;
      oldChildNodes = [].slice.call(oldNode.childNodes);
      for (index = i = ref = splice.index, ref1 = splice.index + splice.removed.length; ref <= ref1 ? i < ref1 : i > ref1; index = ref <= ref1 ? ++i : --i) {
        oldNode.removeChild(oldChildNodes[index]);
      }
      fragment = document.createDocumentFragment();
      for (index = j = ref2 = splice.index, ref3 = splice.index + splice.addedCount; ref2 <= ref3 ? j < ref3 : j > ref3; index = ref2 <= ref3 ? ++j : --j) {
        fragment.appendChild(newChildNodes[index]);
      }
      oldNode.insertBefore(fragment, oldNode.childNodes[splice.index]);
    }
  }, {
    key: 'getChildSplices',
    value: function getChildSplices(oldChildNodes, newChildNodes) {
      var newArr, oldArr;
      oldArr = oldChildNodes.map(this.toUniqueString);
      newArr = newChildNodes.map(this.toUniqueString);
      return _arrayObserver2.default.diff(newArr, oldArr);
    }
  }, {
    key: 'getChildNodesArr',
    value: function getChildNodesArr(realNode) {
      var arr, childNode, i, len, ref;
      arr = [];
      ref = realNode.childNodes;
      for (i = 0, len = ref.length; i < len; i++) {
        childNode = ref[i];
        if (childNode.nodeType === 1) {
          arr.push(childNode.outerHTML);
        } else {
          arr.push('text:' + childNode.nodeValue);
        }
      }
      return arr;
    }
  }, {
    key: 'toUniqueString',
    value: function toUniqueString(realNode) {
      var prefix;
      prefix = realNode.nodeType + ':';
      if (realNode.nodeType === 1) {
        return prefix + realNode.id;
      } else {
        return prefix + realNode.nodeValue;
      }
    }
  }, {
    key: 'updateAttrs',
    value: function updateAttrs(node) {
      var attrName, attribute, i, len, realNode, ref;
      realNode = this.getRealNode(node);
      if (!realNode) {
        return;
      }
      node.eachAttrs(function (name, value) {
        if (!/^\w/.test(name)) {
          return;
        }
        return realNode.setAttribute(name, value);
      });
      ref = [].slice.call(realNode.attributes);
      for (i = 0, len = ref.length; i < len; i++) {
        attribute = ref[i];
        attrName = attribute.name;
        if (!node.hasAttr(attrName)) {
          if (!/^((c|h)\d+)|id$/.test(attrName)) {
            realNode.removeAttribute(attrName);
          }
        }
      }
    }
  }, {
    key: 'updateInput',
    value: function updateInput(node) {
      var realNode;
      realNode = this.getRealNode(node);
      if (!realNode) {
        return;
      }
      if (node.tag === 'select') {
        realNode.selectedIndex = node.index;
        return;
      }
      switch (realNode.type) {
        case 'file':
          if (node.value.length === 0) {
            if (/safari/i.test(navigator.userAgent)) {
              realNode.value = '';
            } else {
              realNode.type = '';
              realNode.type = 'file';
            }
          }
          break;
        case 'checkbox':
          realNode.checked = node.value;
          break;
        default:
          realNode.value = node.value;
          realNode.selectionStart = node.selectionStart;
          realNode.selectionEnd = node.selectionEnd;
      }
    }
  }, {
    key: 'initEventHandlers',
    value: function initEventHandlers() {
      var _this2 = this;

      ['input', 'change', 'mousemove', 'mouseup', 'keyup'].forEach(function (eventName) {
        return _this2.rootReal.addEventListener(eventName, function (event) {
          if (_this2.isInput(event.target)) {
            return _this2.updateVirtualInput(eventName, event.target);
          }
        }, true);
      });
      allEventNames.forEach(function (eventName) {
        return _this2.rootReal.addEventListener(eventName, function (realEvent) {
          return _this2.onDOMEvent(eventName, realEvent);
        }, true);
      });
    }
  }, {
    key: 'isInput',
    value: function isInput(realNode) {
      var tag;
      tag = realNode.tagName.toLowerCase();
      return tag === 'textarea' || tag === 'input';
    }
  }, {
    key: 'onDOMEvent',
    value: function onDOMEvent(eventName, realEvent) {
      var emulatedEvent, lastTouch, realNode, touch;
      if (indexOf.call(touchEventNames, eventName) >= 0) {
        this.isMobile = true;
      }
      if (!this.isMobile) {
        this.emitVirtualEvent(eventName, realEvent, realEvent.target);
        return;
      }
      if (indexOf.call(mouseEventNames, eventName) >= 0) {
        return;
      }
      if (indexOf.call(touchEventNames, eventName) < 0) {
        this.emitVirtualEvent(eventName, realEvent, realEvent.target);
        return;
      }
      switch (eventName) {
        case 'touchstart':
          touch = realEvent.touches[0];
          realNode = touch.target;
          this.lastTouchRealNode = realNode;
          this.lastTouchRealEvent = realEvent;
          this.startTouchRealNode = realNode;
          emulatedEvent = this.touchToMouseEvent(touch, 'mousedown');
          this.emitVirtualEvent('mousedown', emulatedEvent, realNode);
          if (emulatedEvent.defaultPrevented && realEvent.cancelable) {
            realEvent.preventDefault();
          }
          break;
        case 'touchmove':
          touch = realEvent.touches[0];
          realNode = document.elementFromPoint(touch.clientX, touch.clientY);
          if (!realNode) {
            return;
          }
          this.lastTouchRealNode = realNode;
          this.lastTouchRealEvent = realEvent;
          emulatedEvent = this.touchToMouseEvent(touch, 'mousemove');
          this.emitVirtualEvent('mousemove', emulatedEvent, realNode);
          if (emulatedEvent.defaultPrevented && realEvent.cancelable) {
            realEvent.preventDefault();
          }
          break;
        case 'touchend':
          realNode = this.lastTouchRealNode;
          lastTouch = this.lastTouchRealEvent.touches[0];
          emulatedEvent = this.touchToMouseEvent(lastTouch, 'mouseup');
          this.emitVirtualEvent('mouseup', emulatedEvent, realNode);
          if (emulatedEvent.defaultPrevented && realEvent.cancelable) {
            realEvent.preventDefault();
          }
          if (this.touchEndOnTheSameNode()) {
            emulatedEvent = this.touchToMouseEvent(lastTouch, 'click');
            this.emitVirtualEvent('click', emulatedEvent, realNode);
            if (emulatedEvent.defaultPrevented && realEvent.cancelable) {
              realEvent.preventDefault();
            }
          }
      }
    }
  }, {
    key: 'touchToMouseEvent',
    value: function touchToMouseEvent(touch, eventName) {
      var mouseEvent;
      mouseEvent = document.createEvent('MouseEvent');
      mouseEvent.initMouseEvent(eventName, true, true, window, 0, touch.screenX, touch.screenY, touch.clientX, touch.clientY, false, false, false, false, 0, null);
      return mouseEvent;
    }
  }, {
    key: 'touchEndOnTheSameNode',
    value: function touchEndOnTheSameNode() {
      var context;
      context = this.lastTouchRealNode;
      while (context) {
        if (this.startTouchRealNode === context) {
          return true;
        }
        context = context.parentNode;
      }
      return false;
    }
  }, {
    key: 'emitVirtualEvent',
    value: function emitVirtualEvent(eventName, realEvent, realNode) {
      var event, node, srcNode;
      srcNode = this.getVirtualNode(realNode);
      while (realNode && (node = this.getVirtualNode(realNode))) {
        if (node.hasEventHandlers(eventName)) {
          event = _event2.default.emit(eventName, srcNode, node, realEvent);
          if (event.prevented && realEvent.cancelable) {
            realEvent.preventDefault();
          }
          if (event.stopped) {
            realEvent.stopPropagation();
            break;
          }
        }
        realNode = realNode.parentNode;
      }
    }
  }, {
    key: 'updateVirtualInput',
    value: function updateVirtualInput(eventName, realNode) {
      var node, ref;
      if (eventName === 'change' && (ref = realNode.type) !== 'file' && ref !== 'checkbox') {
        return;
      }
      node = this.getVirtualNode(realNode);
      node.setValueFromRealNode(realNode);
    }
  }, {
    key: 'initNode',
    value: function initNode(node) {
      var child, i, j, len, len1, ref, ref1;
      if (!this.getRealNode(node)) {
        return;
      }
      node.init();
      if (node.shadowRoot) {
        ref = node.shadowRoot.children;
        for (i = 0, len = ref.length; i < len; i++) {
          child = ref[i];
          this.initNode(child);
        }
      }
      ref1 = node.children;
      for (j = 0, len1 = ref1.length; j < len1; j++) {
        child = ref1[j];
        this.initNode(child);
      }
    }
  }, {
    key: 'getRealNode',
    value: function getRealNode(virtualNode) {
      return document.getElementById(virtualNode.nodeId);
    }
  }, {
    key: 'getVirtualNode',
    value: function getVirtualNode(realNode) {
      return _dom2.default.getById(realNode.id);
    }
  }]);

  return DOMRenderer;
}();