'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

(function () {
  var HTMLRenderer;

  module.exports = HTMLRenderer = function () {
    _createClass(HTMLRenderer, null, [{
      key: 'render',
      value: function render(node) {
        return new this().render(node);
      }
    }, {
      key: 'renderInnerHTML',
      value: function renderInnerHTML(node) {
        return new this().renderInnerHTML(node);
      }
    }]);

    function HTMLRenderer(root) {
      _classCallCheck(this, HTMLRenderer);

      this.render = this.render.bind(this);
      this.renderNode = this.renderNode.bind(this);
      this.renderElement = this.renderElement.bind(this);
      this.renderContent = this.renderContent.bind(this);
      this.getHost = this.getHost.bind(this);
      this.renderChildren = this.renderChildren.bind(this);
      this.renderInnerHTML = this.renderInnerHTML.bind(this);
      this.renderAttrs = this.renderAttrs.bind(this);
      this.renderComment = this.renderComment.bind(this);
      this.renderTextNode = this.renderTextNode.bind(this);
      this.root = root;
      this.excludeNodes = [];
      return;
    }

    _createClass(HTMLRenderer, [{
      key: 'render',
      value: function render() {
        var node = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.root;

        this.excludeNodes = [];
        return this.renderNode(node);
      }
    }, {
      key: 'renderNode',
      value: function renderNode(node) {
        var needsExclude = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        var html;
        if (this.excludeNodes.indexOf(node) !== -1) {
          return '';
        }
        if (needsExclude) {
          this.excludeNodes.push(node);
        }
        switch (node.nodeType) {
          case 'element':
            if (node.tag === 'content') {
              html = this.renderContent(node);
            } else {
              html = this.renderElement(node);
            }
            break;
          case 'comment':
            html = this.renderComment(node);
            break;
          case 'text':
            html = this.renderTextNode(node);
        }
        return html;
      }
    }, {
      key: 'renderElement',
      value: function renderElement(element) {
        var attrs, innerHTML, tag;
        tag = element.tag;
        attrs = this.renderAttrs(element);
        if (element.shadowRoot) {
          tag = 'ui-' + tag;
          innerHTML = this.renderChildren(element.shadowRoot);
        } else {
          innerHTML = this.renderChildren(element);
        }
        return '<' + tag + attrs + '>' + innerHTML + '</' + tag + '>';
      }
    }, {
      key: 'renderContent',
      value: function renderContent(content) {
        var host, i, innerHTML, len, node, nodes, selector;
        host = this.getHost(content);
        selector = content.attrs.select;
        if (selector) {
          innerHTML = '';
          nodes = host.select('>' + selector);
          for (i = 0, len = nodes.length; i < len; i++) {
            node = nodes[i];
            innerHTML += this.renderNode(node, true);
          }
          return innerHTML;
        }
        return this.renderChildren(host, true);
      }
    }, {
      key: 'getHost',
      value: function getHost(node) {
        while (node) {
          if (node.host) {
            return node.host;
          }
          node = node.parent;
        }
        return null;
      }
    }, {
      key: 'renderChildren',
      value: function renderChildren(element, needsExclude) {
        var child, i, innerHTML, len, ref;
        innerHTML = '';
        ref = element.children;
        for (i = 0, len = ref.length; i < len; i++) {
          child = ref[i];
          innerHTML += this.renderNode(child, needsExclude);
        }
        return innerHTML;
      }
    }, {
      key: 'renderInnerHTML',
      value: function renderInnerHTML(element, needsExclude) {
        return this.renderChildren(element.shadowRoot || element);
      }
    }, {
      key: 'renderAttrs',
      value: function renderAttrs(element) {
        var attrs, host, i, len, name, ref, shadowId, value;
        attrs = '';
        ref = element.attrNames;
        for (i = 0, len = ref.length; i < len; i++) {
          name = ref[i];
          if (!/^\w/.test(name)) {
            continue;
          }
          value = element.attrs[name];
          attrs += ' ' + name + '=\'' + value + '\'';
        }
        if (element.shadowRoot) {
          shadowId = element.shadowRoot.shadowId;
          attrs += ' h' + shadowId;
        }
        host = this.getHost(element);
        if (host) {
          shadowId = host.shadowRoot.shadowId;
          attrs += ' c' + shadowId;
        }
        attrs += ' id=\'' + element.nodeId + '\'';
        return attrs;
      }
    }, {
      key: 'renderComment',
      value: function renderComment(comment) {
        return '';
      }
    }, {
      key: 'renderTextNode',
      value: function renderTextNode(text) {
        return text.value;
      }
    }]);

    return HTMLRenderer;
  }();
}).call(undefined);